@@ -36,8 +36,8 @@ typedef struct _KEYMAP {
     STATUS	(*Func) (void);
 } KEYMAP;
 
-STATIC KEYMAP MetaMap[16] ;
-STATIC KEYMAP Map[33] ; 
+static KEYMAP MetaMap[16] ;
+static KEYMAP Map[33] ; 
 
 /*
 **  Command history structure.
@@ -56,44 +56,44 @@ int		rl_erase;
 int		rl_intr;
 int		rl_kill;
 
-STATIC CHAR		NIL[] = "";
-STATIC CHAR		*Input = NIL;
-STATIC CHAR		*Line = NULL ;
-STATIC CONST CHAR 	*Prompt;
-STATIC CHAR		*Yanked;
+static CHAR		NIL[] = "";
+static CHAR		*Input = NIL;
+static CHAR		*Line = NULL ;
+static const CHAR 	*Prompt;
+static CHAR		*Yanked;
 char		*Screen;
-STATIC char	NEWLINE[]= CRLF;
-/* STATIC char	NEWLINE[]= "\r"; */
-STATIC HISTORY		H;
+static char	NEWLINE[]= CRLF;
+/* static char	NEWLINE[]= "\r"; */
+static HISTORY		H;
 int		rl_quit;
-STATIC int		Repeat;
-STATIC int		End;
-STATIC int		Mark;
-STATIC int		OldPoint;
-STATIC int		Point;
-STATIC int		PushBack;
-STATIC int		Pushed;
-STATIC SIZE_T	Length;
-STATIC SIZE_T	ScreenCount;
+static int		Repeat;
+static int		End;
+static int		Mark;
+static int		OldPoint;
+static int		Point;
+static int		PushBack;
+static int		Pushed;
+static SIZE_T	Length;
+static SIZE_T	ScreenCount;
 SIZE_T			ScreenSize;
-STATIC char		*backspace = NULL ;
-STATIC int		TTYwidth;
-STATIC int		TTYrows;
+static char		*backspace = NULL ;
+static int		TTYwidth;
+static int		TTYrows;
 
 /* Display print 8-bit chars as `M-x' or as the actual 8-bit char? */
 #ifdef NOMETA
-STATIC int		rl_meta_chars = 0 ;
+static int		rl_meta_chars = 0 ;
 #endif
 
 /*
 **  Declarations.
 */
 #ifdef __MSDOS__
-STATIC int 	getakey( CHAR *chr );
+static int 	getakey( CHAR *chr );
 #endif
 
 #ifndef __STDC__
-STATIC CHAR	*editinput();
+static CHAR	*editinput();
 extern int	read();
 extern int	write();
 #endif
@@ -118,8 +118,8 @@ TTYflush( void )
     }
 }
 
-STATIC void
-TTYput( CONST CHAR c )
+static void
+TTYput( const CHAR c )
 {
     Screen[ScreenCount] = c;
     if (++ScreenCount >= ScreenSize - 1) {
@@ -128,15 +128,15 @@ TTYput( CONST CHAR c )
     }
 }
 
-STATIC void
-TTYputs( CONST CHAR *p ) 
+static void
+TTYputs( const CHAR *p ) 
 {
     while (*p)
 	TTYput(*p++);
 	fflush(stdout);
 }
 
-STATIC void
+static void
 TTYshow( CHAR	c )
 {
     if (c == DEL) {
@@ -158,14 +158,14 @@ TTYshow( CHAR	c )
 	TTYput(c);
 }
 
-STATIC void
+static void
 TTYstring( CHAR	*p ) 
 {
     while (*p)
 	TTYshow(*p++);
 }
 
-STATIC UNSI 
+static UNSI 
 TTYget( void )
 {
     CHAR 	c;
@@ -186,19 +186,19 @@ TTYget( void )
 
 #define TTYback() ((backspace != NULL) ? TTYputs((CHAR *)backspace) : TTYput('\b'))
 
-STATIC void
+static void
 TTYbackn( int n)
 {
     while (--n >= 0)
 	TTYback();
 }
 
-STATIC void
+static void
 TTYinfo( void )
 {
     static int		init;
 #if	defined(USE_TERMCAP)
-    CONST char		*term;
+    const char		*term;
     static char		buff[2048];  /* !!!!! */
     char		*bp;
 #endif	/* defined(USE_TERMCAP) */
@@ -282,7 +282,7 @@ columns( int ac, CHAR **av)
     }
 }
 
-STATIC void
+static void
 reposition(void)
 {
     int		i;
@@ -294,7 +294,7 @@ reposition(void)
 	TTYshow(*p);
 }
 
-STATIC void
+static void
 left( STATUS Change)
 {
     TTYback();
@@ -312,7 +312,7 @@ left( STATUS Change)
 	Point--;
 }
 
-STATIC void
+static void
 right( STATUS	Change )
 {
     TTYshow(Line[Point]);
@@ -320,7 +320,7 @@ right( STATUS	Change )
 	Point++;
 }
 
-STATIC STATUS
+static STATUS
 ring_bell( void )
 {
 
@@ -331,7 +331,7 @@ ring_bell( void )
     return CSstay;
 }
 
-STATIC STATUS
+static STATUS
 do_macro( unsigned int	c)
 {
     CHAR		name[4];
@@ -348,7 +348,7 @@ do_macro( unsigned int	c)
     return CSstay;
 }
 
-STATIC STATUS
+static STATUS
 do_forward( STATUS move )
 {
     int		i;
@@ -372,7 +372,7 @@ do_forward( STATUS move )
     return CSstay;
 }
 
-STATIC STATUS
+static STATUS
 do_case( CASE type)
 {
     int		i;
@@ -400,19 +400,19 @@ do_case( CASE type)
     return CSstay;
 }
 
-STATIC STATUS
+static STATUS
 case_down_word( void )
 {
     return do_case(TOlower);
 }
 
-STATIC STATUS
+static STATUS
 case_up_word( void )
 {
     return do_case(TOupper);
 }
 
-STATIC void
+static void
 ceol( void )
 {
     int		extras;
@@ -438,7 +438,7 @@ ceol( void )
 	TTYback();
 }
 
-STATIC void
+static void
 clear_line( void )
 {
     Point = ( 0 - strlen(Prompt) ); 
@@ -449,7 +449,7 @@ clear_line( void )
     Line[0] = '\0';
 }
 
-STATIC STATUS
+static STATUS
 insert_string( CHAR *p )
 {
     SIZE_T	len;
@@ -484,19 +484,19 @@ insert_string( CHAR *p )
 }
 
 
-STATIC CHAR *
+static CHAR *
 next_hist( void )
 {
     return H.Pos >= H.Size - 1 ? NULL : H.Lines[++H.Pos];
 }
 
-STATIC CHAR *
+static CHAR *
 prev_hist( void )
 {
     return H.Pos == 0 ? NULL : H.Lines[--H.Pos];
 }
 
-STATIC STATUS
+static STATUS
 do_insert_hist(CHAR *p)
 {
     if (p == NULL)
@@ -508,7 +508,7 @@ do_insert_hist(CHAR *p)
     return insert_string(p);
 }
 
-STATIC STATUS
+static STATUS
 do_hist( CHAR	*(*move)(void) )
 {
     CHAR	*p;
@@ -522,25 +522,25 @@ do_hist( CHAR	*(*move)(void) )
     return do_insert_hist(p);
 }
 
-STATIC STATUS
+static STATUS
 h_next(void)
 {
     return do_hist(next_hist);
 }
 
-STATIC STATUS
+static STATUS
 h_prev(void)
 {
     return do_hist(prev_hist);
 }
 
-STATIC STATUS
+static STATUS
 h_first(void)
 {
     return do_insert_hist(H.Lines[H.Pos = 0]);
 }
 
-STATIC STATUS
+static STATUS
 h_last(void)
 {
     return do_insert_hist(H.Lines[H.Pos = H.Size - 1]);
@@ -549,8 +549,8 @@ h_last(void)
 /*
 **  Return zero if pat appears as a substring in text.
 */
-STATIC int
-substrcmp(CONST char *text, CONST char *pat, SIZE_T len)
+static int
+substrcmp(const char *text, const char *pat, SIZE_T len)
 {
     CHAR	c;
 
@@ -562,13 +562,13 @@ substrcmp(CONST char *text, CONST char *pat, SIZE_T len)
     return 1;
 }
 
-STATIC CHAR *
+static CHAR *
 search_hist( CHAR *search, CHAR *(*move)(void) )
 {
     static CHAR	*old_search;
     int		len;
     int		pos;
-    int		(*match)(CONST char *, CONST char *, SIZE_T );
+    int		(*match)(const char *, const char *, SIZE_T );
     char	*pat;
 
     /* Save or get remembered search pattern. */
@@ -601,11 +601,11 @@ search_hist( CHAR *search, CHAR *(*move)(void) )
     return NULL;
 }
 
-STATIC STATUS
+static STATUS
 h_search( void )
 {
     static int	Searching;
-    CONST CHAR	*old_prompt;
+    const CHAR	*old_prompt;
     CHAR	*(*move)(void);
     CHAR	*p;
 
@@ -627,7 +627,7 @@ h_search( void )
     return do_insert_hist(p);
 }
 
-STATIC STATUS
+static STATUS
 fd_char( void )
 {
     int		i;
@@ -641,7 +641,7 @@ fd_char( void )
     return CSstay;
 }
 
-STATIC void
+static void
 save_yank( int begin, int i)
 {
     if (Yanked) {
@@ -658,7 +658,7 @@ save_yank( int begin, int i)
     }
 }
 
-STATIC STATUS
+static STATUS
 delete_string( int count)
 {
     int		i;
@@ -702,7 +702,7 @@ delete_string( int count)
     return CSmove;
 }
 
-STATIC STATUS
+static STATUS
 bk_char( void )
 {
     int		i;
@@ -717,7 +717,7 @@ bk_char( void )
     return CSstay;
 }
 
-STATIC STATUS
+static STATUS
 bk_del_char( void )
 {
     int		i;
@@ -732,13 +732,13 @@ bk_del_char( void )
     return delete_string(i);
 }
 
-STATIC STATUS
+static STATUS
 del_char(void)
 {
     return delete_string(Repeat == NO_ARG ? 1 : Repeat);
 }
 
-STATIC STATUS
+static STATUS
 redisplay(void)
 {
     TTYputs((CHAR *)NEWLINE);
@@ -747,7 +747,7 @@ redisplay(void)
     return CSmove;
 }
 
-STATIC STATUS
+static STATUS
 kill_line(void)
 {
     int		i;
@@ -773,7 +773,7 @@ kill_line(void)
     return CSstay;
 }
 
-STATIC STATUS
+static STATUS
 insert_char(int c)
 {
     STATUS	s;
@@ -799,7 +799,7 @@ insert_char(int c)
     return s;
 }
 
-STATIC STATUS
+static STATUS
 meta(void)
 {
     UNSI        	c;
@@ -909,7 +909,7 @@ meta(void)
     return ring_bell();
 }
 
-STATIC STATUS
+static STATUS
 emacs( unsigned int c)
 {
     STATUS		s;
@@ -931,7 +931,7 @@ emacs( unsigned int c)
     return s;
 }
 
-STATIC STATUS
+static STATUS
 TTYspecial( unsigned int c) 
 {
 #ifdef NOMETA
@@ -959,7 +959,7 @@ TTYspecial( unsigned int c)
     return CSdispatch;
 }
 
-STATIC CHAR *
+static CHAR *
 editinput(void)
 {
     extern char *luxptr ;
@@ -1006,7 +1006,7 @@ editinput(void)
     return NULL;
 }
 
-STATIC void
+static void
 hist_add( CHAR *p )
 {
     int		i;
@@ -1038,7 +1038,7 @@ rl_reset_terminal( char *p )
 char *
 readline( char *prompt, int scrollflag)
 {
-    CHAR	*line;
+    CHAR	*line_p;
 
     if (Line == NULL) {
 	Length = MEM_INC;
@@ -1053,9 +1053,9 @@ readline( char *prompt, int scrollflag)
     Screen = NEW(char, ScreenSize);
     Prompt = (CHAR *) (prompt != NULL ? prompt : (char *) NIL ) ; 
     TTYputs(Prompt);
-    line = editinput();
-    if ( line != NULL  ) {
-	line = (CHAR *)strdup((char *)line);
+    line_p = editinput();
+    if ( line_p != NULL  ) {
+	line_p = (CHAR *)strdup((char *)line_p);
 	if(scrollflag)
 	TTYputs((CHAR *)NEWLINE);
 	else
@@ -1065,7 +1065,7 @@ readline( char *prompt, int scrollflag)
     rl_ttyset(1);
     DISPOSE(Screen);
     DISPOSE(H.Lines[--H.Size]);
-    return (char *)line;
+    return (char *)line_p;
 }
 
 void
@@ -1082,7 +1082,7 @@ add_history( char *p)
 }
 
 
-STATIC STATUS
+static STATUS
 beg_line(void)
 {
     if (Point) {
@@ -1092,7 +1092,7 @@ beg_line(void)
     return CSstay;
 }
 
-STATIC STATUS
+static STATUS
 end_line(void)
 {
     if (Point != End) {
@@ -1106,7 +1106,7 @@ end_line(void)
 **  Move back to the beginning of the current word and return an
 **  allocated copy of it.
 */
-STATIC CHAR *
+static CHAR *
 find_word(void)
 {
     static char	SEPS[] = "#;&|^$=`'{}()<>\n\t ";
@@ -1124,18 +1124,18 @@ find_word(void)
     return new;
 }
 
-STATIC STATUS
+static STATUS
 c_complete(void)
 {
     CHAR	*p;
-    CHAR	*word;
+    CHAR	*word_p;
     int		unique;
     STATUS	s;
 
-    word = find_word();
-    p = (CHAR *)rl_complete((char *)word, &unique);
-    if (word)
-	DISPOSE(word);
+    word_p = find_word();
+    p = (CHAR *)rl_complete((char *)word_p, &unique);
+    if (word_p)
+	DISPOSE(word_p);
     if (p && *p) {
 	s = insert_string(p);
 	if (!unique)
@@ -1146,17 +1146,17 @@ c_complete(void)
     return ring_bell();
 }
 
-STATIC STATUS
+static STATUS
 c_possible(void)
 {
     CHAR	**av;
-    CHAR	*word;
+    CHAR	*word_p;
     int		ac;
 
-    word = find_word();
-    ac = rl_list_possib((char *)word, (char ***)&av);
-    if (word)
-	DISPOSE(word);
+    word_p = find_word();
+    ac = rl_list_possib((char *)word_p, (char ***)&av);
+    if (word_p)
+	DISPOSE(word_p);
     if (ac) {
 	columns(ac, av);
 	while (--ac >= 0)
@@ -1167,14 +1167,14 @@ c_possible(void)
     return ring_bell();
 }
 
-STATIC STATUS
+static STATUS
 accept_line(void)
 {
     Line[End] = '\0'; 
     return CSdone;
 }
 
-STATIC STATUS
+static STATUS
 transpose(void)
 {
     CHAR	c;
@@ -1192,7 +1192,7 @@ transpose(void)
     return CSstay;
 }
 
-STATIC STATUS
+static STATUS
 quote(void)
 {
     UNSI    c;
@@ -1205,7 +1205,7 @@ quote(void)
       return  insert_char((int)c );
 }
 
-STATIC STATUS
+static STATUS
 wipe(void)
 {
     int		i;
@@ -1223,14 +1223,14 @@ wipe(void)
     return delete_string(Mark - Point);
 }
 
-STATIC STATUS
+static STATUS
 mk_set(void)
 {
     Mark = Point;
     return CSstay;
 }
 
-STATIC STATUS
+static STATUS
 exchange(void)
 {
     UNSI	c;
@@ -1250,7 +1250,7 @@ exchange(void)
     return CSstay;
 }
 
-STATIC STATUS
+static STATUS
 yank(void)
 {
     if (Yanked && *Yanked)
@@ -1258,7 +1258,7 @@ yank(void)
     return CSstay;
 }
 
-STATIC STATUS
+static STATUS
 copy_region(void)
 {
     if (Mark > End)
@@ -1272,7 +1272,7 @@ copy_region(void)
     return CSstay;
 }
 
-STATIC STATUS
+static STATUS
 move_to_char(void)
 {
     UNSI 		c;
@@ -1289,13 +1289,13 @@ move_to_char(void)
     return CSstay;
 }
 
-STATIC STATUS
+static STATUS
 fd_word(void)
 {
     return do_forward(CSmove);
 }
 
-STATIC STATUS
+static STATUS
 fd_kill_word(void)
 {
     int		i;
@@ -1309,7 +1309,7 @@ fd_kill_word(void)
     return CSstay;
 }
 
-STATIC STATUS
+static STATUS
 bk_word(void)
 {
     int		i;
@@ -1330,7 +1330,7 @@ bk_word(void)
     return CSstay;
 }
 
-STATIC STATUS
+static STATUS
 bk_kill_word(void)
 {
     (void)bk_word();
@@ -1339,8 +1339,8 @@ bk_kill_word(void)
     return CSstay;
 }
 
-STATIC int
-argify( CHAR *line, CHAR ***avp )
+static int
+argify( CHAR *line_p, CHAR ***avp )
 {
     CHAR	*c;
     CHAR	**p;
@@ -1352,7 +1352,7 @@ argify( CHAR *line, CHAR ***avp )
     if ((*avp = p = NEW(CHAR*, i))== NULL)
 	 return 0;
 
-    for (c = line; isspace(*c); c++)
+    for (c = line_p; isspace(*c); c++)
 	continue;
     if (*c == '\n' || *c == '\0')
 	return 0;
@@ -1383,7 +1383,7 @@ argify( CHAR *line, CHAR ***avp )
     return ac;
 }
 
-STATIC STATUS
+static STATUS
 last_argument(void)
 {
     CHAR	**av;
@@ -1411,7 +1411,7 @@ last_argument(void)
 
 
 #ifdef __MSDOS__
-STATIC int
+static int
 getakey( CHAR *chr ) {
 
 	static int pushed = 0 ;
