@@ -3,11 +3,14 @@
 **  Editline system header file for Unix.
 */
 
+#define strdup xc_strdup
+extern char *xc_strdup(const char *s);
 #define CRLF		"\r\n"
-#define FORWARD		STATIC
 
 #include <sys/types.h>
 #include <sys/stat.h>
+/* extern int ioctl(int fildes, int request, ...);*/
+#include <sys/ioctl.h>
 
 #if	defined(USE_DIRENT)
 #include <dirent.h>
