@@ -58,14 +58,14 @@ typedef enum {
 	S_Send_NAK,
 	S_SenD_ACK,
 	S_SEnd_ENQ,
-	S_Resend_Packets,
+	S_Resend_Packets
 } Sender_Action;
 
 extern short cr_add;
 extern void cl_line();
 static char S_Buffer[2064], R_Buffer[2064], tdir[32];
 char	Name[SM_BUFF];
-static unchar Mask[32];
+static unsigned char Mask[32];
 static unsigned Checksum;
 static	Ch,				/* last char read from remote */
 		Quoting,		/* quoting level requested by the user */
@@ -82,13 +82,13 @@ static long	already_have, data, total_read, total_sent,
 			fsize, carriage_return;
 static time_t start;
 
-struct {
+static struct {
 	int Seq;
 	int PackeT_Size;
 	char *packet;
 } Pending[Send_Ahead_Buffers];
 
-extern ushort crc_xmodem_tab[256];
+extern unsigned short crc_xmodem_tab[256];
 
 static void
 init_check()
@@ -360,7 +360,7 @@ Init()
 static void
 Xmit_Packet(Size, Seq, Packet)
 int Size, Seq;
-unchar *Packet;
+unsigned char *Packet;
 {
 	register I;
 
@@ -646,7 +646,7 @@ int Size;
 
 	Next_Seq=Seq_Num=(Seq_Num+1)%10;
 	Pending[Next].Seq=Next_Seq;
-	Pending[Next].packet=strdup(S_Buffer);
+	Pending[Next].packet=xc_strdup(S_Buffer);
 	Pending[Next].PackeT_Size=Size;
 	Packets_Btwn_ACKs=0;
 
@@ -655,13 +655,13 @@ int Size;
 	return SUCCESS;
 }
 
-static ulong
+static unsigned long
 cnvAtoL(ptr)
 char *ptr;
 {
-	ushort sign=FALSE;
+	unsigned short sign=FALSE;
 	char ch;
-	ulong result=0;
+	unsigned long result=0;
 
 	ch= *ptr++;
 
@@ -679,7 +679,7 @@ char *ptr;
 static
 char *cnvLtoA(ptr, n)
 char *ptr;
-ulong n;
+unsigned long n;
 {
 	char tmp1[11], *tmp2=tmp1;
 
@@ -792,12 +792,12 @@ char *buf;
 register int want;
 {
 	register c;
-	int read=0;
+	int read_ct=0;
 
 	while (want--)
 		switch(c=getc(fp)){
 		case EOF:
-			return read;
+			return read_ct;
 
 		case '\n':
 			if (cr_add&&textmode&&Last_Chr!='\r')
@@ -807,10 +807,10 @@ register int want;
 
 		default:
 			Last_Chr= *buf++ =c;
-			read++;
+			read_ct++;
 		}
 
-	return read;
+	return read_ct;
 }
 
 static int
@@ -1081,13 +1081,13 @@ Send_File()
 #define My_Upload_Resume	0
 #define My_File_Information	1
 
-static unchar Quote_Level_Select_Low[]={
+static unsigned char Quote_Level_Select_Low[]={
 	1, 3, 3, 0, 3, 0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
 	0, 0, 3, 0, 3, 0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3
 };
 
 
-static unchar QuotE_Level_select_Hi[]={
+static unsigned char QuotE_Level_select_Hi[]={
 	3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
 	3, 2, 3, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3
 };
@@ -1103,7 +1103,7 @@ static int
 Plus_Respond()
 {
 	int Status, temp_window_size, temp_method, temp_size, MaskByte, Bit, i;
-	unchar Estimated_Quote_Level=0;
+	unsigned char Estimated_Quote_Level=0;
 
 	S_Buffer[0]='+';
 	S_Buffer[1]=My_Send_Window_Size;
