@@ -7,14 +7,14 @@
 #include <unistd.h>
 #include <sys/timeb.h>
 #include <time.h>
-#define NODEBUG 1	/* prevents xc.h from defining DEBUG */
+#define NODEBUG 1 /* prevents unwanted recursion in this module */
 #include "xc.h"
 
 #if DEBUG
 static FILE *dfp;
 
 void
-dbglog()
+dbglog P_((void))
 {
 	time_t todnow;
 
@@ -45,6 +45,29 @@ FILE *stream;
 	return(fputs(s,stream));
 }
 
+#if _POSIX_SOURCE || __STDC__
+#include <stdarg.h>
+int
+Fprintf(FILE *stream, const char *format, ...)
+{
+    int retcode;
+    va_list arg;
+
+    if (!capture && dfp) {
+	va_start(arg, format);
+	vfprintf(dfp, format, arg);
+	va_end(arg);
+    }
+
+    va_start(arg, format);
+    retcode = vfprintf(stream, format, arg);
+    va_end(arg);
+
+    return (retcode);
+}
+
+#else /* not posix */
+
 int
 Fprintf(stream, format, a, b, c, d, e, f, g)
 FILE *stream;
@@ -56,4 +79,7 @@ long a, b, c, d, e, f, g;
 	return(fprintf(stream,format,a,b,c,d,e,f,g));
 }
 
-#endif
+#endif /*_POSIX_SOURCE || __STDC__*/
+
+#endif /* DEBUG */
+
