@@ -9,206 +9,70 @@
 #include <unistd.h>
 #include <signal.h>
 #include <ctype.h>
-#include <termio.h>
 #include <setjmp.h>
+#include <sys/wait.h>
+#include <getopt.h>
 #include "xc.h"
 
-#define Resume_Not_Allowed	1
+#define Resume_Not_Allowed  1
 
-#ifdef RLINE
-	char *luxptr = NULL ;
-	extern char *readline();
-	extern void *rl_initialize();	
+static pid_t honeychile;   /* for fork pid */
+
+#if XC_PLUS
+#if HAVE_RS232FLOW
+short rtscts = TRUE;  /* RTS/CTS flow control */
+#endif
+static int sentkill = FALSE;    	
+short dosmode	= TRUE;  /* Translate DOS chars to latin1/vt102 */
+short delmode	= TRUE;  /* Map DEL to BS */
+short auto_zm   = TRUE;  /* Allow Automagic ZModem downloads */
+short nowait    = FALSE; /* When TRUE this will disable call waiting */
+short tone	= TRUE;  /* Flag for pulse or touch tone dialing */
+char rzcmd[SM_BUFF];     /* Buffer for auto_zmodem command */
 #endif	
 
-#ifdef linux
-#include <sys/wait.h>
-static	pid_t honeychile,   /* for fork pid */
-			sentkill ;    	
-	short
-	dosmode = 	TRUE,	/* Translate DOS chars to latin1/vt102 */
-	delmode =	TRUE,	/* Map DEL to BS */
-  	rtscts = TRUE,		/* RTS/CTS flow control */
-  	auto_zm = TRUE,		/* Allow Automagic ZModem downloads */
-    nowait = FALSE,     /* When TRUE this will disable call waiting */
-    tone = TRUE;        /* Flag for pulse or touch tone dialing */
-	char rzcmd[SM_BUFF];/* Buffer for auto_zmodem command */
-#endif	
-	short
-	autoflag =	FALSE,	/* Automatic capturing */
+short	autoflag =	FALSE,	/* Automatic capturing */
 	cismode =	FALSE,	/* Automatic response to CIS "ENQ" */
 	cr_add =	TRUE,	/* Add cr to nl in B+ uploads */
+#if HALF_DUPLEX
 	hdplxflag =	FALSE,	/* Half-duplex mode */
- 	menuflag =	FALSE,	/* Show mini-menu */
+#endif
 	nl2cr =		TRUE,	/* Map nl to cr when transmitting ASCII */
 	reterm =	FALSE,	/* Jumping into terminal mode */
-	statflag =	FALSE,	/* Flag for status display */
 	eofflag =	FALSE;	/* Flag to quit a script */
-void s_cis(), s_set(), s_exit(); 
-int s_shell();
-char Msg[SM_BUFF]; 
-#ifdef CHGDIR
-char directory[60];
-#endif
-char BS, LK;
+
+static short statflag =	FALSE;	/* Flag for status display */
+
 FILE *tfp;
-struct TERMIOS newmode, oldmode, sigmode;
-static char	*statfmt = "\r\t\t%-8s %25s %s\r\n",
-#ifdef linux
-			version[]="@(#)XC 4.3 JPRadley  |    Linux p1 Oct 30, 1994    |",
+struct termios newmode, oldmode, sigmode;
+
+             char Msg[SM_BUFF], BS ;
+static       char LK, directory[SM_BUFF], oldshell[SM_BUFF];
+static const char * const statfmt = "\r\t\t%-8s %25s %s\r\n",
+#if XC_PLUS
+ version[]="@(#)XC 4.3 JPRadley  |   xcplus p2 Nov 24, 1996    |",
 #else
-			version[]="@(#)XC 4.3 JPRadley 11 Sep 1993",
+ version[]="@(#)XC 4.3 JPRadley 11 Sep 1993",
 #endif
-			oldshell[SM_BUFF],
-			*babble[] = {
-				"\r\nUsage: xc [-l device] [-s file | -t | -p | -w ]",
-				"\t-l device\tUse 'device' as the modem port",
-				"\t-s script\tExecute 'script' immediately",
-				"\t-t\t\tEnter terminal mode immediately",
-#ifdef linux				
-				"\t-p\t\tUse pulse dialing instead of tone dialing",
-				"\t-w\t\tDisable call waiting",
+	* const babble[] = {
+		"\r\nUsage: xc [-l device] [-s file | -t | -p | -w ]",
+		"\t-l device\tUse 'device' as the modem port",
+		"\t-s script\tExecute 'script' immediately",
+		"\t-t\t\tEnter terminal mode immediately",
+#if XC_PLUS				
+		"\t-p\t\tUse pulse dialing instead of tone dialing",
+		"\t-w\t\tDisable call waiting",
 #endif	
 				NULL
-			};
-static void s_script(), s_xmodem(), s_term(), s_help(), s_dial(), puttake(),
-	SET_proto(), SET_cr(), SET_cis(), SET_nl(), SET_xon(), SET_xcape(),
-	SET_bps(), SET_autocapt(), SET_cfile(), SET_pfile(),
-#ifndef linux
-	SET_menu, SET_dir();
-#else /* linux */
-	SET_del(), SET_dos(), SET_rtscts(), SET_azm(), SET_zmcmd(), SET_hdplx();
-#endif
-extern short scriptflag;
-extern void B_Transfer(), dbglog(), mattach(), terminal(), xreceive(), xsend(),
-	get_ttype(), unlock_tty();
-jmp_buf erret;			/* non-local error return */
+		};
 
-struct kw {				/* Used by command parsing routines */
+jmp_buf erret;		/* non-local error return */
+
+struct kw {		/* Used by command parsing routines */
 	char *keyword;
 	void (*rtn)();
 };
 
-static struct kw cmds[] = {
-	{"c",		s_cis},
-	{"cis",		s_cis},
-	{"s",		s_script},
-	{"script",	s_script},
-	{"h",		(void (*)())hangup},
-	{"hangup",	(void (*)())hangup},
-	{"bindings",	show_bindings},
-#ifdef linux
-	{"ro",		s_xmodem},
-	{"rx",		s_xmodem},
-	{"rg",		s_xmodem},
-	{"so",		s_xmodem},
-	{"sx",		s_xmodem},
-	{"sg",		s_xmodem},
-	{"ry",		s_xmodem},
-	{"rbg",		s_xmodem},
-	{"ryg",		s_xmodem},
-	{"sy",		s_xmodem},
-	{"sbg",		s_xmodem},
-	{"syg",		s_xmodem},
-#endif
-	{"rb",		s_xmodem},
-	{"rt",		s_xmodem},
-	{"sb",		s_xmodem},
-	{"st",		s_xmodem},
-	{"set",		s_set},
-	{"t",		s_term},
-	{"term",	s_term},
-	{"d",		s_dial},
-	{"dial",	s_dial},
-	{"q",		s_exit},
-	{"quit",	s_exit},
-	{"exit",	s_exit},
-	{"x",		s_exit},
-	{"!",  (void(*)()) s_shell},
-	{"!!", (void(*)()) s_shell},
-	{"$",  (void(*)()) s_shell},
-	{"%p",		puttake},
-	{"%t",		puttake},
-	{"help",	s_help},
-	{"?",		s_help},
-	{NULL,	0}
-};
-
-static struct kw setlist[] = {
-	{"auto",	SET_autocapt},
-	{"baud",	SET_bps},
-	{"bps",		SET_bps},
-	{"cfile",	SET_cfile},
-	{"cis",		SET_cis},
-	{"cr",		SET_cr},
-	{"hdplx",	SET_hdplx},
-#ifdef CHGDIR
-	{"dir",		SET_dir},
-#endif
-#ifndef linux	
-	{"menu",	SET_menu},
-#endif	
-	{"nl",		SET_nl},
-	{"pfile",	SET_pfile},
-	{"proto",	SET_proto},
-	{"escape",	SET_xcape},
-	{"xcape",	SET_xcape},
-	{"xon",		SET_xon},
-	{"xoff",	SET_xon},
-#ifdef linux
-  	{"rtscts",	SET_rtscts},
-	{"del2bs",	SET_del},
-	{"msdos",	SET_dos}, 
-  	{"autozm",	SET_azm},
-  	{"zmcmd",	SET_zmcmd},
-#endif	
-	{ NULL,	0}
-};
-
-/* Print the status of the program */
-static void
-status()
-{
-	struct kw *ptr;
-	char p[30];
-	void (*fct)() = 0;
-
-	statflag = TRUE;
-
-	cls();
-	cur_off();
-	sprintf(p,"Modem Port: %s",mport(NULL));
-	drawline(0, 0, CO);
-#ifdef linux	
-	ttgoto(1, 1);
-	sprintf(Msg,"%s%27s",&version[4], p);
-#else
-	ttgoto(1, 9);	
-	sprintf(Msg,"%-29s%29s",&version[4], p);
-#endif	
-	S;
-	drawline(2, 0, CO);
-	ttgoto(3, 0);
-	fprintf(tfp, statfmt, "Keyword", "Description", "Status");
-	fprintf(tfp, statfmt, "--------", "-------------------------", "-----------");
-
-	for (ptr = setlist; ptr->keyword; ptr++)
-		if (ptr->rtn != fct){
-			fct = ptr->rtn;
-			(*fct)();
-		}
-
-#ifdef linux
-	ttgoto(21, 5);
- 	S1("[d]ial directory  [t]erminal mode  [q]uit  [s]cript  [?]help");
-	ttgoto(22, 71);
-#else
-	ttgoto(18, 25);
-	S1("Type \"help\" or ? for help");
-#endif	
-	statflag = FALSE;
-	cur_on();
-}
 
 /* Catch a signal and jump to main. Reset signal and do a longjmp */
 static RETSIGTYPE
@@ -217,7 +81,7 @@ int junk;
 {
 	if (! isatty(2))
 		hangup(),
-		s_exit();
+		s_exit(junk);
 
 	S2("XC: Interrupt");
 
@@ -229,171 +93,13 @@ int junk;
 static void
 usage()
 {
-	char **ptr;
+	const char * const *ptr;
 
 	for (ptr = babble; *ptr; ptr++)
 		fprintf(tfp, "%s\r\n", *ptr);
 }
 
-int
-main(argc, argv)
-int argc;
-char **argv;
-{
-	char *script = NULL ;
-	extern char *optarg;
-	int c;
 
-	struct kw *ptr;
-	tfp = stderr;
-	if (isatty(2))
-		get_ttype();
-#ifdef linux
-	tcgetattr( 0, &oldmode);
-#else
-	ioctl(0, TCGETA, &oldmode);	/* get current tty mode	*/
-#endif
-	/* trap for SIGHUP and SIGTERM, make sure LCKfile gets killed */
-	signal(SIGHUP,(RETSIGTYPE *)s_exit);
-	signal(SIGTERM,(RETSIGTYPE *)s_exit);
-
-	newmode = oldmode;
-
-	newmode.c_iflag &= ~(IXON | IXOFF | IXANY);
-	newmode.c_lflag &= ~(ICANON | ISIG | ECHO);
-	newmode.c_oflag = 0;
-	newmode.c_cc[VMIN] = 1;
-	newmode.c_cc[VTIME] = 1;
-	BS = newmode.c_cc[VERASE];
-	LK = newmode.c_cc[VKILL];
-
-	sigmode = newmode;
-	sigmode.c_lflag |= ISIG;
-
-	oldshell[0] = '\0';	/* set last command to blank */
-	if (setjmp(erret))	/* set error handler to exit */
-		exit(0);		/*  while parsing command line */
-	signal(SIGINT,catch);	/* catch break & quit signals/keys */
-	signal(SIGQUIT,catch);
-	
-#ifdef RLINE	
-	rl_initialize();
-#endif
-	default_bindings();
-
-#ifdef linux
-  	strcpy(rzcmd,"rz -y\0"); /* auto_zm */
-	while ((c = getopt(argc, argv, "s:l:tpw")) != -1)
-#else
-	while ((c = getopt(argc, argv, "s:l:t")) != -1)
-#endif	
-		switch (c){
-		case 'l':	/* set modem port name */
-			mport(optarg);
-			break;
-		case 's':	/* Execute SCRIPT file */
-			script = optarg;
-			break;
-		case 't':	/* jump into terminal mode */
-			reterm = TRUE;
-			break;
-#ifdef linux			
-        case 'p':   /* use pulse dialing  */
-            tone = FALSE;
-            break;       
-        case 'w':  /* disable call waiting  */
-            nowait = TRUE ; 
-            break; 
-#endif			
-		default:	/* Bad command .. print help */
-			usage();
-			exit(1);
-		}
-
-	setuid(geteuid());
-	setgid(getegid());
-
-	mopen();	/* opens and configures modem port, or exits */
-
-	setuid(getuid());
-	setgid(getgid());
-#ifdef CHGDIR
-	getcwd(directory, 64);
-#endif
-	do_script(STARTUP);
-
-#if DEBUG
-	dbglog();
-#endif
-
-	if (!script)
-		status();
-
-	for (;;){
-		setjmp(erret);
-		signal(SIGQUIT,(RETSIGTYPE *)s_exit);
-		mode(SIGMODE);
-
-		if (script)
-			do_script(script),
-			script = NULL,
-			reterm = TRUE;
-
-		if (reterm && isatty(2)){
-			s_term();
-			continue;
-		}
-
-#ifdef linux
-		/* no reverse video and no vt102 line graphics */
-		fprintf(tfp, "%c\033[m", (char) 15 );
-		memset( line, 0, SM_BUFF);
-#endif                                        
-#ifdef RLINE
-lloop:
-#endif
-		fputc('\r',tfp);
-		fputc('\n',tfp);
-#ifndef linux		
- 		if (menuflag)
-			fputc('\t',tfp),
- 			S1("[d]ial directory  [t]erminal mode  [q]uit  [s]cript  [?]help");
-#endif			
-#ifndef RLINE
-		show(-1,"<XC>");
-		fputc(' ',tfp);
-		lptr = line;
-		getline();
-		fputc('\r',tfp),
-		fputc('\n',tfp);
-#else	
-		lptr = readline( XCPROMPT ) ;
-		strcpy( line, lptr );
-		add_history( lptr);
-		free( lptr) ;
-		lptr = line ;
-		if( ! *lptr ){
-				status();
-				goto lloop;
-		}
-#endif		
-		getword();
-		lc_word(word);
-		if (word[0] == '\0')		/* If blank line... reprompt */
-			continue;
-
-		for (ptr = cmds; ptr->keyword; ptr++)
-			if (!strcmp(word, ptr->keyword))
-				break;
-
-		if (ptr->keyword)
-			(*ptr->rtn)();
-		else
-			sprintf(Msg,"Unrecognized command: %s",word),
-			S;
-	}
-	return SUCCESS ;
-}
 
 static void
 s_script()
@@ -427,7 +133,7 @@ s_xmodem()
 	{
 		y_flag = TRUE ;
 		if (d == 's')
-			ysend(e);
+			ysend();
 		else
 			yreceive(e);
 	}
@@ -447,6 +153,24 @@ s_xmodem()
 	xc_setproto(oldproto);
 }
 
+void
+s_cis()
+{
+	char oldproto[4];
+
+	strcpy(oldproto, protocol);
+
+	xc_setflow(FALSE);
+	xc_setproto("8N1");
+	mode(SIGMODE);
+
+	B_Transfer();
+
+	reterm = TRUE;
+	xc_setflow(flowflag);
+	xc_setproto(oldproto);
+}
+
 static void
 s_term()
 {
@@ -467,55 +191,42 @@ s_dial()
 	s_cis();
 }
 
-void
-s_cis()
+#if XC_PLUS && _POSIX_SOURCE
+/* send SIGTERM to shell child process */
+static void
+sig_int(int junk)
 {
-	char oldproto[4];
-
-	strcpy(oldproto, protocol);
-
-	xc_setflow(FALSE);
-	xc_setproto("8N1");
-	mode(SIGMODE);
-
-	B_Transfer();
-
-	reterm = TRUE;
-	xc_setflow(flowflag);
-	xc_setproto(oldproto);
-}
-
-#ifdef linux
-void
-siggyint(int junk)
-{
-		signal( SIGINT, SIG_IGN );
-		if( honeychile > 0 ){
-				kill( -(honeychile), SIGTERM);
-				sleep(2);
-				kill( -(honeychile), SIGKILL);
-				sentkill = 1 ;
-		}
+	signal( SIGINT, SIG_IGN );
+	if( honeychile > 0 ){
+		kill( -(honeychile), SIGINT);
+		sleep(2);
+		if(kill(honeychile,0) == 0 ){
+			kill( -(honeychile), SIGTERM);
+			sleep(2);
+			kill( -(honeychile), SIGKILL);
+		}	
+		sentkill = TRUE ;
+	}
 }
 #endif
 
 int
 s_shell()
 {
+#if NOSHELL
+	return(0);
+#else
 	int stat_loc = 0;
 	char c = word[0];
 	static char *shell = NULL ;
 	RETSIGTYPE (*oldvec)();
 
-#ifdef linux
+#if XC_PLUS
 	struct termios savedterm ; 
-	sentkill = 0 ;
-	tcgetattr(0, &savedterm )	;
-	honeychile = 0 ;
+	tcgetattr(STDIN_FILENO, &savedterm );
+	sentkill = FALSE ;
 #endif	
-#if NOSHELL
-	return(0);
-#endif
+	honeychile = 0 ;
 	if (word[0] == word[1])
 		strcpy(wptr = word, oldshell);
 	else {
@@ -528,30 +239,19 @@ s_shell()
 		shell = getenv("SHELL");
 		if (!shell)
 			shell = "/bin/sh";
-#ifdef linux			
-  		else
-  			if ( !strchr(shell,'/')) {
-  				shell = (char *) alloca(strlen(getenv("SHELL") + 6));
-  				strcpy(shell,"/bin/");
-  				strcat(shell,getenv("SHELL"));
-			}
-#endif				
 	}
 
 	fputc('\r',tfp),
 	fputc('\n',tfp);
 	mode(OLDMODE);
 
-#ifdef linux
-	if ( (honeychile = forkem()) == 0 ){
-#else		
-	if (!forkem()){
-#endif		
+	if ( (honeychile = forkem()) == 0 )
+	{
 		if (c == '$'){	/* Attach modem to stdin, stdout */
 			setpgid( 0, 0 );
 			mattach();
 		}
-		signal(SIGCLD,SIG_DFL);
+		signal(SIGCHLD,SIG_DFL);
 		signal(SIGINT, SIG_DFL );
 		signal(SIGQUIT,SIG_DFL);
 		if (word[0] == '\0')
@@ -562,61 +262,90 @@ s_shell()
 		exit(2);
 	}
 
-#ifdef linux	
+#if !XC_PLUS || !_POSIX_SOURCE
+	oldvec = signal(SIGINT,SIG_IGN);
+	wait(&stat_loc);
+	signal(SIGINT,oldvec);
+
+	strcpy(oldshell, wptr);
+	return(!!stat_loc); 
+
+#else /*XC_PLUS and _POSIX_SOURCE*/
 	if( honeychile > 0 ){
-		if( c == '$' )
-			setpgid( honeychile, honeychile );
-		oldvec = signal(SIGINT,siggyint);
-		waitpid(honeychile, &stat_loc, 0) ; 
+		if( c == '$' ){
+#if _POSIX_SOURCE
+			setpgid(honeychile, honeychile);
+#endif
+			oldvec = signal(SIGINT, sig_int);
+		} else {
+			oldvec = signal(SIGINT, SIG_IGN);
+		}
+		waitpid(honeychile, &stat_loc, 0) ;
 	} else {
 		printf("shell fork() error\n");
 		exit(1);
 	}
-#else	
-	oldvec = signal(SIGINT,SIG_IGN);
-	wait(&stat_loc);
-#endif	
-	strcpy(oldshell, wptr);
 	signal(SIGINT,oldvec);
-#ifdef linux	
+	strcpy(oldshell, wptr);
 	tcsetattr(0, TCSANOW, &savedterm )	;
-	if( WIFEXITED( stat_loc ) )
-		if( !WEXITSTATUS( stat_loc ) && !sentkill )
-			return( 0 ) ;
-	if( sentkill )		
-		fprintf(stderr, "\n\rcaught SIGINT\n\r");	
-	else	
-		fprintf(stderr, "\n\r\nstatus = %#x   return code = %#x\n\r",
-		stat_loc, WEXITSTATUS(stat_loc) ); sleep(1);
-	return( 1 )	;
-#else
-	return(!!stat_loc); 
+	if( WIFEXITED( stat_loc ) && !WEXITSTATUS( stat_loc ) && !sentkill )
+		return( 0 ) ;
+	if( sentkill ) {
+		int watch_dog = XC_WATCH_DOG ;
+		fprintf(stderr, "\n\rCaught SIGINT\n\r");	
+		fprintf(stderr, "Purging input buffer . . .\n\r");	
+
+		/* 
+		We use a watch dog so that we don't hang forever
+		if the other end refuses to shut up. This will 
+		return control to the user who can take manual  
+		action such as sending a string of ^X . 
+		*/
+
+		do{
+#if _POSIX_SOURCE
+			purge();
+			sleep(1); 
 #endif
+			watch_dog-- ;
+		}
+		while(watch_dog && readbyte(1) != -1 ) ;
+		if(! watch_dog ) {
+		    canit();
+		    fprintf(stderr, BAD_PROGRAM_MSG );
+		    sleep(2);
+		}
+		sendbyte(13);
+	} else {	
+		fprintf(stderr, "\n\r\nstatus = %#x   return code = %#x\n\r",
+		stat_loc, WEXITSTATUS(stat_loc) ); 
+	}
+	sleep(1);
+	return( 1 )	;
+#endif /*XC_PLUS*/
+#endif /*not NOSHELL*/
 }
 
-static char	*cmdlist[] = {
+static const char * const cmdlist[] = {
 	"\tXC Command Summary",
 	"",
-	"\tc",
-	"\tcis\t\tInitiate CIS B+ File Transfer (Upload and Download)",
+	"\tc|cis\t\tInitiate CIS B+ File Transfer (Upload and Download)",
 	"",
-	"\td",
-	"\tdial\t\tDialing directory",
+	"\td|dial\t\tDialing directory",
 	"",
-	"\tx",
-	"\tq",
-	"\texit",
-	"\tquit\t\tExit XC",
+	"\tx|q|exit|quit\tExit XC",
 	"",
-	"\th",
-	"\thangup\t\tHang up the modem",
+	"\th|hangup\tHang up the modem",
 	"",
-#ifdef linux
-	"\trx file\t\tXMODEM receive file 'file' (binary mode)",
-	"\trt file\t\tXMODEM receive file 'file' (Ascii mode)",
-	"\tsx file...\tXMODEM send file 'file' (binary mode 128 byte packets)",
-	"\tso file...\tXMODEM send file 'file' (binary mode 1K byte packets)",
-	"\tst file...\tXMODEM send file 'file' (Ascii mode)",
+#if XC_PLUS
+	"\trx file\t\tXmodem receive file 'file' (binary mode)",
+	"\try \t\tYmodem receive file (binary, file name passed by sender)", 
+	"\tryg\t\tYmodem receive file 'G' mode (binary, streaming ymodem)",
+	"\tsx file\t\tXmodem send file 'file' (binary mode 128 byte packets)",
+	"\tso file\t\tXmodem send file 'file' (binary mode 1K byte packets)",
+	"\tsy file\t\tYmodem send file 'file' (binary mode)",
+	"\trt file\t\tXmodem receive file 'file' (obsolete ascii mode)",
+	"\tst file\t\tXmodem send file 'file' (obsolete ascii mode)",
 #else
 	"\trb file\t\tXMODEM receive file 'file' (binary mode)",
 	"\trt file\t\tXMODEM receive file 'file' (Ascii mode)",
@@ -649,7 +378,7 @@ static char	*cmdlist[] = {
 	"",
 	"\t?",
 	"\thelp\t\tPrint (this) help text",
-	"",
+	"\014",  /* FORM FEED */
 	"\tSET Keywords:",
 	"",
 	"\tset\t\t\tDisplay current XC status",
@@ -669,14 +398,14 @@ static char	*cmdlist[] = {
 	"\tset xcape char",
 	"\tset escape char\t\tSet the Terminal mode escape character",
 	"",
+#if HALF_DUPLEX
 	"\tset hdplx on\t\tSet half-duplex mode",
 	"\tset hdplx off\t\tUnset half-duplex mode (use full-duplex)",
 	"",
-#ifdef CHGDIR
+#endif
 	"\tset dir name\tChange current directory",
 	"",
-#endif
-#ifndef linux	
+#if !XC_PLUS	
  	"\tset menu on|off\t\tDo|Don't show mini-menu before XC prompt",
 	"",
 #endif	
@@ -691,84 +420,25 @@ static char	*cmdlist[] = {
 	"\tset xon on|off",
 	"\tset xoff on|off\t\tSet|Unset XON/XOFF flow control",
 	"",
-#ifdef linux	
+#if XC_PLUS	
+#if HAVE_RS232FLOW
   	"\tset rtscts on|off\t\tSet/Reset RTS/CTS flow control\n",
+#endif	
 	"\tset del2bs on|off\tmap|unmap Delete character to Backspace",
 	"\tset msdos on|off\ttranslate DOS characters to Linux chars.",
   	"\tset autozm on|off\t\tSet/Reset Automatic ZModem download\n",
   	"\tset zmcmd command\t\tSet command used to invoke ZModem Receive\n",
 	"",
 #endif	
-	"",
+	"\014", /* FORM FEED */
 	NULL };
 
-static void
-s_help()
-{
-	char **ptr = cmdlist;
-	int curline = 0;
-
-	mode(OLDMODE);
-	cls();
-	cur_off();
-	for ( ; *ptr; ptr++) {
-		if (**ptr != '') {
-			if (curline >= LI-2){
-				S0("PRESS ENTER");
-				getline();
-				cls();
-				curline = 0;
-			}
-			fprintf(tfp, "%s\r\n", *ptr);
-			curline++;
-		} else {
-			S0("PRESS ENTER");
-			getline();
-			cls();
-			curline = 0;
-		}
-	}
-	show_bindings();
-	S0("PRESS ENTER");
-	getline();
-	cls();
-	status();
-}
-
-void
-s_set()
-{
-	struct kw *ptr;
-
-	getword();
-
-	if (word[0] == '\0' && !scriptflag){
-		status();
-		return;
-	} else if (word[0] == '\0'){
-		S1("SET keyword requires an argument");
-		eofflag++;
-		return;
-	}
-
-	lc_word(word);
-
-	for (ptr = setlist; ptr->keyword; ptr++)
-		if (!strcmp(ptr->keyword, word)){
-			(*ptr->rtn)();
-			return;
-		}
-
-	sprintf(Msg,"Invalid SET keyword: %s", word);
-	S;
-	eofflag++;
-}
 
 void
 set_onoff(flag)
 short *flag;
 {
-	char *ptr = strdup(word);
+	char *ptr = xc_strdup(word);
 
 	uc_word(ptr);
 	getword();
@@ -808,8 +478,7 @@ SET_proto()
 		S;
 }
 
-#ifdef CHGDIR
-static
+static void
 SET_dir()
 {
 	if (statflag){
@@ -823,10 +492,9 @@ SET_dir()
 		sprintf(Msg,"Can't move to %s", directory);
 
 	if (!scriptflag)
-		sprintf(Msg, "We are in %s", directory);
+		sprintf(Msg, "%sWe are in %s", CE, directory);
 	S;
 }
-#endif
 
 static void
 SET_cr()
@@ -902,7 +570,7 @@ SET_cis()
 		S;
 }
 
-#ifdef linux
+#if XC_PLUS
 static void
 SET_del()
 {
@@ -988,6 +656,7 @@ SET_azm()
  		}
 }
   
+#if HAVE_RS232FLOW
  static void
  SET_rtscts()
  {
@@ -1003,7 +672,8 @@ SET_azm()
    if (!scriptflag)
  	fprintf(tfp, "<<Hardware Flow control is %s>>\r\n",rtscts?"ON":"OFF");
  }
-#endif
+#endif /* HAVE_RS232FLOW */
+#endif /* XC_PLUS */
 
 static void
 SET_xon()
@@ -1044,6 +714,7 @@ SET_bps()
 		S;
 }
 
+#if HALF_DUPLEX
 static void
 SET_hdplx()
 {
@@ -1059,8 +730,10 @@ SET_hdplx()
 		sprintf(Msg,"Half-duplex Mode is %s", hdplxflag ? "ON" : "OFF"),
 		S;
 }
+#endif
 
-#ifndef linux
+#if !XC_PLUS
+static short menuflag =	FALSE;	/* Show mini-menu */
 static void
 SET_menu()
 {
@@ -1213,7 +886,7 @@ puttake()
 			send_string(wrkbuf);		/* send command to remote shell */
 			while (readbyte(3) != '\n')	/* discard up to the \n in wrkbuf */
 				;
-			while ((Ch=readbyte(0)) != -1	/* while chars are being sent */
+			while ((Ch=readbyte(4)) != -1	/* while chars are being sent */
 					 && Ch != DLE)			/* and we haven't seen our DLE */
 				fputc(Ch,fp);
 			fclose(fp);
@@ -1225,15 +898,351 @@ puttake()
 }
 
 void
-s_exit()
+s_exit(int junk)
 {
 	signal(SIGHUP,SIG_IGN);
 	signal(SIGINT,SIG_IGN);
 	signal(SIGQUIT,SIG_IGN);
 	signal(SIGTERM,SIG_IGN);
-
 	mode(OLDMODE);
 	unlock_tty();
+	reset_crt();
+	exit(EXIT_SUCCESS);
+}
 
-	exit(0);
+
+
+
+
+
+
+
+
+static struct kw setlist[] = {
+	{"auto",	SET_autocapt},
+	{"baud",	SET_bps},
+	{"bps",		SET_bps},
+	{"cfile",	SET_cfile},
+	{"cis",		SET_cis},
+	{"cr",		SET_cr},
+#if HALF_DUPLEX
+	{"hdplx",	SET_hdplx},
+#endif
+	{"dir",		SET_dir},
+#if !XC_PLUS	
+	{"menu",	SET_menu},
+#endif	
+	{"nl",		SET_nl},
+	{"pfile",	SET_pfile},
+	{"proto",	SET_proto},
+	{"escape",	SET_xcape},
+	{"xcape",	SET_xcape},
+	{"xon",		SET_xon},
+	{"xoff",	SET_xon},
+#if XC_PLUS
+#if HAVE_RS232FLOW
+  	{"rtscts",	SET_rtscts},
+#endif
+	{"del2bs",	SET_del},
+	{"msdos",	SET_dos}, 
+  	{"autozm",	SET_azm},
+  	{"zmcmd",	SET_zmcmd},
+#endif	
+	{ NULL,	0}
+} ;
+
+/* Print the status of the program */
+static void
+status()
+{
+	struct kw *ptr;
+	char p[30];
+	void (*fct)() = 0;
+
+	statflag = TRUE;
+
+	cls();
+	cur_off();
+	sprintf(p,"Modem Port: %s",mport(NULL));
+	drawline(0, 0, CO);
+#if XC_PLUS	
+	ttgoto(1, 1);
+	sprintf(Msg,"%s%27s",&version[4], p);
+#else
+	ttgoto(1, 9);	
+	sprintf(Msg,"%-29s%29s",&version[4], p);
+#endif	
+	S;
+	drawline(2, 0, CO);
+	ttgoto(3, 0);
+	fprintf(tfp, statfmt, "Keyword", "Description", "Status");
+	fprintf(tfp, statfmt, "--------", "-------------------------", "-----------");
+
+	for (ptr = setlist; ptr->keyword; ptr++)
+		if (ptr->rtn != fct){
+			fct = ptr->rtn;
+			(*fct)();
+		}
+
+#if XC_PLUS
+	ttgoto(21, 5);
+ 	S1("[d]ial directory  [t]erminal mode  [q]uit  [s]cript  [?]help");
+	ttgoto(22, 71);
+#else
+	ttgoto(18, 25);
+	S1("Type \"help\" or ? for help");
+#endif	
+	statflag = FALSE;
+	cur_on();
+}
+
+void
+s_set()
+{
+	struct kw *ptr;
+
+	getword();
+
+	if (word[0] == '\0' && !scriptflag){
+		status();
+		return;
+	} else if (word[0] == '\0'){
+		S1("SET keyword requires an argument");
+		eofflag++;
+		return;
+	}
+
+	lc_word(word);
+
+	for (ptr = setlist; ptr->keyword; ptr++)
+		if (!strcmp(ptr->keyword, word)){
+			(*ptr->rtn)();
+			return;
+		}
+
+	sprintf(Msg,"Invalid SET keyword: %s", word);
+	S;
+	eofflag++;
+}
+
+static void
+s_help()
+{
+	const char * const *ptr = cmdlist;
+	int curline = 0;
+
+	mode(OLDMODE);
+	cls();
+	cur_off();
+	for ( ; *ptr; ptr++) {
+		if (**ptr != '\014') {  /* FORM FEED */
+			if (curline >= LI-2){
+				S0("PRESS ENTER");
+				getline();
+				cls();
+				curline = 0;
+			}
+			fprintf(tfp, "%s\r\n", *ptr);
+			curline++;
+		} else {
+			S0("PRESS ENTER");
+			getline();
+			cls();
+			curline = 0;
+		}
+	}
+	show_bindings();
+	S0("PRESS ENTER");
+	getline();
+	cls();
+	status();
+}
+
+static struct kw cmds[] = {
+	{"c",		s_cis},
+	{"cis",		s_cis},
+	{"s",		s_script},
+	{"script",	s_script},
+	{"h",		(void (*)())hangup},
+	{"hangup",	(void (*)())hangup},
+	{"bindings",	show_bindings},
+#if XC_PLUS
+	{"ro",		s_xmodem},
+	{"rx",		s_xmodem},
+	{"rg",		s_xmodem},
+	{"so",		s_xmodem},
+	{"sx",		s_xmodem},
+	{"sg",		s_xmodem},
+	{"ry",		s_xmodem},
+	{"rbg",		s_xmodem},
+	{"ryg",		s_xmodem},
+	{"sy",		s_xmodem},
+	{"sbg",		s_xmodem},
+	{"syg",		s_xmodem},
+#endif
+	{"rb",		s_xmodem},
+	{"rt",		s_xmodem},
+	{"sb",		s_xmodem},
+	{"st",		s_xmodem},
+	{"set",		s_set},
+	{"t",		s_term},
+	{"term",	s_term},
+	{"d",		s_dial},
+	{"dial",	s_dial},
+	{"q",		s_exit},
+	{"quit",	s_exit},
+	{"exit",	s_exit},
+	{"x",		s_exit},
+	{"!",  (void(*)()) s_shell},
+	{"!!", (void(*)()) s_shell},
+	{"$",  (void(*)()) s_shell},
+	{"%p",		puttake},
+	{"%t",		puttake},
+	{"help",	s_help},
+	{"?",		s_help},
+	{NULL,	0}
+};
+
+int
+main(argc, argv)
+int argc;
+char **argv;
+{
+	char *script = NULL ;
+	extern char *optarg;
+	int c;
+
+	struct kw *ptr;
+	tfp = stderr;
+	if (isatty(2))
+		get_ttype();
+	tcgetattr(STDIN_FILENO, &oldmode);	/* get current tty mode	*/
+	/* trap for SIGHUP and SIGTERM, make sure LCKfile gets killed */
+	signal(SIGHUP,s_exit);
+	signal(SIGTERM,s_exit);
+
+	newmode = oldmode;
+
+	newmode.c_iflag &= ~(IXON | IXOFF | IXANY);
+	newmode.c_lflag &= ~(ICANON | ISIG | ECHO);
+	newmode.c_oflag = 0;
+	newmode.c_cc[VMIN] = 1;
+	newmode.c_cc[VTIME] = 1;
+	BS = newmode.c_cc[VERASE];
+	LK = newmode.c_cc[VKILL];
+
+	sigmode = newmode;
+	sigmode.c_lflag |= ISIG;
+
+	oldshell[0] = '\0';	/* set last command to blank */
+	if (setjmp(erret))	/* set error handler to exit */
+		exit(0);		/*  while parsing command line */
+	/*signal(SIGINT,catch);*/	/* catch break & quit signals/keys */
+	signal(SIGINT,SIG_IGN);
+	signal(SIGQUIT,catch);
+	
+#if RLINE	
+	rl_initialize();
+#endif
+	default_bindings();
+
+#if XC_PLUS
+  	strcpy(rzcmd,"rz -y\0"); /* auto_zm */
+	while ((c = getopt(argc, argv, "s:l:tpw")) != -1)
+#else
+	while ((c = getopt(argc, argv, "s:l:t")) != -1)
+#endif	
+		switch (c){
+		case 'l':	/* set modem port name */
+			mport(optarg);
+			break;
+		case 's':	/* Execute SCRIPT file */
+			script = optarg;
+			break;
+		case 't':	/* jump into terminal mode */
+			reterm = TRUE;
+			break;
+#if XC_PLUS			
+        case 'p':   /* use pulse dialing  */
+            tone = FALSE;
+            break;       
+        case 'w':  /* disable call waiting  */
+            nowait = TRUE ; 
+            break; 
+#endif			
+		default:	/* Bad command .. print help */
+			usage();
+			exit(1);
+		}
+
+	setuid(geteuid());
+	setgid(getegid());
+
+	mopen();	/* opens and configures modem port, or exits */
+
+	setuid(getuid());
+	setgid(getgid());
+	getcwd(directory, SM_BUFF-2);
+	do_script(STARTUP);
+
+#if DEBUG
+	dbglog();
+#endif
+
+	if (!script)
+		status();
+
+	for (;;){
+		setjmp(erret);
+		signal(SIGQUIT,s_exit);
+		mode(SIGMODE);
+
+		if (script) {
+			do_script(script);
+			script = NULL;
+			reterm = TRUE;
+		}
+
+		if (reterm && isatty(2)){
+			s_term();
+			continue;
+		}
+		reset_crt();
+
+#if RLINE
+		for(;;) {
+#endif
+		fputc('\r',tfp);
+		fputc('\n',tfp);
+#if !XC_PLUS		
+ 		if (menuflag){
+			fputc('\t',tfp);
+ 			S1("[d]ial directory  [t]erminal mode  [q]uit  [s]cript  [?]help");
+		}
+#endif			
+		prompt_user(XCPROMPT);
+		getline();
+#if RLINE
+		if( *lptr == '\0' )
+		    status();
+		else
+		    break ;
+		} /*  END for(;;) */
+#endif		
+		getword();
+		lc_word(word);
+		if (word[0] == '\0')		/* If blank line... reprompt */
+			continue;
+
+		for (ptr = cmds; ptr->keyword; ptr++)
+			if (!strcmp(word, ptr->keyword))
+				break;
+
+		if (ptr->keyword)
+			(*ptr->rtn)();
+		else
+			sprintf(Msg,"Unrecognized command: %s",word),
+			S;
+	}
+	return SUCCESS ;
 }
