@@ -19,24 +19,25 @@
 #include <sys/stat.h>
 #include "xc.h"
 
-jmp_buf			here;
-extern FILE		*cfp;
-short			ttyflag, debugflag, linkflag, scriptflag,
-				mrbstart,	/* ring buffer start pointer */
-				mrbcount,	/* ring buffer counter */
-				BREAK = 0;	/* a hook for a later 'trap' keyword */
+extern FILE	*cfp;
 extern short	eofflag;
-extern int	redial(), xmitbrk();
-extern void	s_set(), s_exit(), divert(), set_onoff(), xcdial();
+static jmp_buf 	here;
+short		linkflag, scriptflag;
 
-int				my_escape = 1;	/* Control-A; resettable at run-time */
+static short	ttyflag, debugflag, 
+		mrbstart,	/* ring buffer start pointer */
+		mrbcount,	/* ring buffer counter */
+		BREAK = 0;	/* a hook for a later 'trap' keyword */
 
-char			mringbuf[LG_BUFF];	/* ring buffer for modem input */
-static			S_abort();
-static void		unsetall(), S_bombout(), S_call();
-static char		*NO_ARG = "%s command must have an argument";
+int my_escape = 1; /* Control-A; resettable at run-time */
 
-static bindfunc_t   find_function(); /* bindfunc_t : see xc.h */
+static char mringbuf[LG_BUFF];	/* ring buffer for modem input */
+
+static int  S_abort P_((void));
+static void unsetall P_((void)), S_bombout P_((void)), S_call P_((char *sname));
+static char *NO_ARG = "%s command must have an argument";
+
+static bindfunc_t find_function P_((char *fname)); /* bindfunc_t : see xc.h */
 
 typedef struct bindstruct {
 	int					bs_c;		/* Character prefix */
@@ -64,10 +65,9 @@ static bindstr_t function_list[] = {
 	{HLPCHAR, "hlpchar", "Display terminal mode key bindings"},
 	{BRKCHAR, "brkchar", "Send modem BREAK signal"},
 	{EMITSTR, "emitstr", "Emit string"},
-#ifdef linux
-	{CLRGRAF, "clrgraf", "Turn off VT102 line graphics"},		
-#endif	
-	{DOSCRPT, "doscrpt", "Execute script file"}
+	{CLRGRAF, "clrgraf", "Reset crt and turn off line graphics"},		
+	{DOSCRPT, "doscrpt", "Execute script file"},
+	{SENDCAN, "sendcan", "Send CAN string to cancel transfer"}
 };
 
 #define FUNCTION_COUNT (sizeof(function_list) / sizeof(function_list[0]))
@@ -86,7 +86,7 @@ void
 do_script(file)
 char *file;
 {
-	RETSIGTYPE (*oldvec)();
+	RETSIGTYPE (*oldvec) P_((int));
 
 	capture = eofflag = FALSE;
 	ttyflag = scriptflag = TRUE;
@@ -371,7 +371,7 @@ char *string;
 
 	ptr->bs_c = c;
 	ptr->bs_function = function;
-	ptr->bs_string = strdup(string);
+	ptr->bs_string = xc_strdup(string);
 
 	/* The following is an insertion sort to ensure that the bindings are
 	   stored in ascending sequence by key code.  This makes
@@ -537,7 +537,7 @@ static VAR	*Varlist = NIL(VAR);	/* top of variable list */
 static VAR	*Lastvar = NIL(VAR);	/* bottom of variable list */
 
 /* Valid variable name characters */
-unchar 
+static unsigned char 
 OKname[]= {
 	 /* control characters */
 	  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
@@ -911,7 +911,7 @@ char **pcptr;			/* address of script program counter */
 				}
 			}
 			bqcmd[i] = '\0';
-			signal(SIGCLD,SIG_DFL);
+			signal(SIGCHLD,SIG_DFL);
 			if (!(bqpipe=popen(bqcmd,"r"))){
 				sprintf(Msg,"%s: cannot create pipe",bqcmd);
 				S_abort();
@@ -1372,7 +1372,7 @@ static int	nest_while,		/* number of nested loops */
 
 static time_t deadline;		/* deadline for 'timeout' keyword */
 
-jmp_buf env;				/* cell for environment, setjmp */
+static jmp_buf env;		/* cell for environment, setjmp */
 
 #define CMDNEST		8		/* max number of nested scripts */
 #define PARSNEST	50		/* max number of nested calls to parser */
@@ -1925,13 +1925,12 @@ default_bindings()
 	bind_key('/', HLPCHAR, "");
 	bind_key('?', HLPCHAR, "");
 	bind_key('b', BRKCHAR, "");
+	bind_key('c', SENDCAN, "");
 	bind_key('d', DIALCHR, "");
 	bind_key('f', DIVCHAR, "");
 	bind_key('h', HUPCHAR, "");
 	bind_key('n', CAPTEND, "");
-#ifdef linux	
 	bind_key('o', CLRGRAF, "");
-#endif	
 	bind_key('q', QUITCHR, "");
 	bind_key('s', SCRPCHR, "");
 	bind_key('x', ENDCHAR, "");
@@ -1968,7 +1967,7 @@ void
 show_bindings()
 {
 	binding_t *ptr = first_binding;
-	char *escape_str = strdup(unctrl(my_escape));
+	char *escape_str = xc_strdup(unctrl(my_escape));
 	int curline = 0;
 
 	cls();
