@@ -5,6 +5,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include <sys/types.h>
 #include <utime.h>
 #include <sys/stat.h>
@@ -12,6 +13,8 @@
 #include <setjmp.h>
 #include "xc.h"
 
+#define WANTCRG 'G'
+#define WANTCRC 'C'
 #define FALLBACK 16
 #define CPMEOF  032		/* ^Z */
 #define OK       0
@@ -21,18 +24,18 @@
 #define RETRYMAX 10
 #define Maxtime 10
 #define SECSIZ  1025
-#define Resume_Not_Allowed      1
-#define RUpdcrc(c,crc) (crc_xmodem_tab[((crc>>8)&0xff)]^(crc<<8)^c)&0xffff
-#define TUpdcrc(c,crc) (crc_xmodem_tab[((crc>>8)^c)&0xff]^(crc<<8))
-
-short int badline = FALSE;
-short int g_flag  = FALSE;
-short int y_flag  = FALSE;
-short int y_done  = FALSE;
-short int smallhdr = FALSE;
+#define Resume_Not_Allowed 1
+#define RxUpdcrc(c,crc) (crc_xmodem_tab[((crc>>8)&0xff)]^(crc<<8)^c)&0xffff
+#define TxUpdcrc(c,crc) (crc_xmodem_tab[((crc>>8)^c)&0xff]^(crc<<8))
+#include <assert.h>   
+static short int smallhdr = FALSE;
+static short int badline  = FALSE;
+static short int g_flag   = FALSE;
+static short int y_done   = FALSE;
+       short int y_flag   = FALSE;  /* global used in xcmain.c */
 
 /* define storage for info from decoding ymodem rx header */
-struct yinfo {
+static struct yinfo {
 	char *fnam ;
 	long int fsiz ;
 	time_t fdat ;
@@ -40,7 +43,7 @@ struct yinfo {
 }  phdr ;
 
 /* crc_xmodem_tab calculated by Mark G. Mendel, Network Systems Corporation */
-ushort crc_xmodem_tab[256] =
+unsigned short crc_xmodem_tab[256] =
 {
     0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50a5, 0x60c6, 0x70e7,
     0x8108, 0x9129, 0xa14a, 0xb16b, 0xc18c, 0xd1ad, 0xe1ce, 0xf1ef,
@@ -87,22 +90,17 @@ static short firstsec,		/* first sector of file or not? */
 
 static char wcbuf[SECSIZ];	/* Ward Christensen sector buffer */
 static jmp_buf our_env;
-static RETSIGTYPE (*oldvec)();
 static char *p, sendchar;
-static char syncem(void);
-static int yparse(void);
-void set_dp( const char *p, const time_t dt, const unsigned long prm);
 
-/*      send 10 CAN's to try to get the other end to shut up */
-static void
-canit ()
+
+/* send cancel string to get the other end to shut up */
+void
+canit P_((void))
 {
-    int i;
-
-    for (i = 0; i < 20; i++)
-	sendbyte (CAN);
-    for (i = 0; i < 20; i++)
-	sendbyte (8); /* backspaces to clear CAN characters */
+	char canistr[] = {
+	 24,24,24,24,24,24,24,24,24,24,8,8,8,8,8,8,8,8,8,8,0
+	};
+	send_string(canistr);
 }
 
 static RETSIGTYPE
@@ -115,12 +113,13 @@ int junk;
     longjmp (our_env, 1);
 }
 
+
 /*      fill the CP/M sector buffer from the UNIX file
         do text adjustments if necessary
         return 1 if more sectors are to be read, or 0 if this is the last
 */
 static int
-getsec ()
+getsec P_((void))
 {
     int i;
     register c = 0 ;
@@ -197,6 +196,51 @@ getsec ()
       }
 }
 
+/* automatically negotiate crc or checksum reception */
+
+static char
+syncem P_((void))
+{
+    int ct = 2;
+    char c;
+
+    purge ();
+
+    while (ct)
+      {	  /* try to negotiate a CRC transfer */
+	  sendbyte ( g_flag ? WANTCRG : WANTCRC );
+	  c = readbyte (10);
+	  if (c != CAN && c != SOH && c != STX)
+	      ct--;
+	  else
+	    {
+		sendchar = ACK;
+		crcheck = g_flag ? WANTCRG : WANTCRC ;
+		return (c);
+	    }
+	    if ( g_flag ) { ct = 2, g_flag = FALSE ;} /* try again without "G" */
+      }
+    ct = 2;
+    purge ();
+    while (ct)
+      {				/* maybe we can initiate a checksum transfer */
+	  sendbyte (NAK);
+	  c = readbyte (10);
+	  if (c != CAN && c != SOH && c != STX)
+	      ct--;
+	  else
+	    {
+		sendchar = ACK;
+		crcheck = FALSE;
+		return (c);
+	    }
+      }
+    purge ();
+    return (TIMEOUT);
+
+}
+
+
 /*      wcgetsec() inputs an XMODEM "sector".
         This routine returns the sector number encountered, or ERROR if a valid
         sector is not received or CAN received; or WCEOT if EOT sector.
@@ -253,17 +297,17 @@ wcgetsec (maxtime)
 			    if ((c = readbyte (3)) == TIMEOUT)
 				goto timeout;
 			    wcbuf[j] = c;
-			    oldcrc = RUpdcrc (c, oldcrc);
+			    oldcrc = RxUpdcrc (c, oldcrc);
 			    checksum += c;
 			}
 		      if ((c = readbyte (3)) < 0)
 			  goto timeout;
 		      if (crcheck)
 			{
-			    oldcrc = RUpdcrc (c, oldcrc);
+			    oldcrc = RxUpdcrc (c, oldcrc);
 			    if ((c = readbyte (3)) == TIMEOUT)
 				goto timeout;
-			    if (RUpdcrc (c, oldcrc))
+			    if (RxUpdcrc (c, oldcrc))
 			      {
 				  show(2,"CRC error     ");
 				  if(g_flag) {
@@ -315,7 +359,7 @@ gquit:
 }
 
 static int
-putsec ()
+putsec P_((void))
 {
     int i;
     register c;
@@ -342,88 +386,30 @@ putsec ()
     return (0);
 }
 
-/* Receive a file using XMODEM protocol */
-static int
-wcrx ()
+static void
+set_dp( name_fl, time_fl, perm_fl )
+   const char *name_fl;
+   const time_t time_fl;
+   const unsigned long perm_fl;
 {
-    register sectnum, sectcurr;
+	struct utimbuf utb ;
 
-    if ( !y_flag ){
-    	/* open file for xmodem reception */
-    	strcpy (Name, word);
-    	xfp = QueryCreate (Resume_Not_Allowed) ;
-    	if ( xfp == NULL)
-		return (ERROR); 
-    }
-
-    firstsec = TRUE;
-    sectnum = y_flag ? -1 : 0 ;
-    sendchar = g_flag? WANTCRG : WANTCRC ;
-    sync = FALSE;
-    y_done = FALSE;
-    crcheck = TRUE;
-    show(2,"Sync...");
-
-    while (TRUE)
-      {
-	  if (badline)
-	      purge ();
-	  if ( g_flag == FALSE && sync )
-	      sendbyte (sendchar);
-	  if ( y_done )
-	      { cl_line(); return OK; }
-	  if ( sectnum == 0 && y_flag )
-	      sendbyte((crcheck ? crcheck : NAK));
-	  sectcurr = wcgetsec (Maxtime);
-
-	  if (sectcurr == ((sectnum + 1) & 0xff))
-	    {
-		if ( firstsec ){
-			if ( y_flag ){	/* open file for rx */
-				if (yparse() == ERROR ) return ERROR;
-			}
-			else{ 
-				fprintf(stderr,"\r%s\n\r", Name );
-			}
-			firstsec = FALSE ;
+	if ( name_fl != NULL ) {
+		if ( time_fl != 0 ) {
+			utb.actime = utb.modtime = time_fl ;
+			if ( utime( name_fl, &utb ))
+				perror(name_fl);
 		}
-		sectnum++;
-	        if (sectnum == 1)
-	            show (1,(ksecsize == 128 ? "Sender using 128 byte packets   ":"Sender using 1K byte packets    "));
-		if ( sectnum > 0 )
-		    putsec ();
-		fprintf (tfp, "Received sector #%d\r", sectnum);
-		sendchar = ACK;
-		continue;
-	    }
+		if ( perm_fl != 0 ) {
+			if ( chmod( name_fl, (mode_t) perm_fl ))
+				perror(name_fl);
+		}
+	}
 
-	  if (sectcurr == (sectnum & 0xff))
-	    {
-		sprintf (Msg, "Received duplicate sector #%d", sectnum);
-		show(2,Msg);
-		sendchar = ACK;
-		continue;
-	    }
-
-	  fclose(xfp);
-	  if( y_flag ) set_dp( phdr.fnam, phdr.fdat, phdr.fprm );
-
-	  if (sectcurr == WCEOT)
-	    {
-		show( 1,"File received OK                ");
-		sendbyte (ACK);
-		return (OK);
-	    }
-
-	  if (sectcurr == ERROR)
-	      return (ERROR);
-
-	  sprintf (Msg, "Sync error ... expected %d(%d), got %d",
-		   (sectnum + 1) & 0xff, sectnum, sectcurr);
-	  show(1,Msg);
-	  return (ERROR);
-      }
 }
+
+
+
 /*      wcputsec outputs a Ward Christensen type sector.
         it returns OK or ERROR
 */
@@ -432,102 +418,145 @@ static int
 wcputsec (sectnum)
      int sectnum;
 {
-    register ushort oldcrc;
-    int checksum, j, c, attempts, errct = 0;
+  unsigned short oldcrc;
+  int checksum, j, c, attempts, errct = 0;
 
-    oldcrc = checksum = 0;
-    for (j = 0; j < (smallhdr ? 128 : ksecsize); j++)
-      {
-	  c = wcbuf[j];
-	  oldcrc = TUpdcrc (c, oldcrc);
-	  checksum += c;
-      }
+  oldcrc = checksum = 0;
+  for (j = 0; j < (smallhdr ? 128 : ksecsize); j++)
+    {
+      c = wcbuf[j];
+      oldcrc = TxUpdcrc (c, oldcrc);
+      checksum += c;
+    }
 
-    for (attempts = 0; attempts < RETRYMAX; attempts++)
-      {
-	  purge();
-	  sendbyte ( smallhdr ? SOH : stx_soh);
-	  sendbyte (sectnum);
-	  sendbyte (-sectnum - 1);
-	  for (j = 0; j < ( smallhdr ? 128 : ksecsize); j++)
-	      sendbyte (wcbuf[j]);
-	  if ( firstsec == TRUE || badline == TRUE)
-	      purge ();
-	  if (crcheck)
-	    {
-		sendbyte ((int) (oldcrc >> 8));
-		sendbyte ((int) oldcrc);
+  for (attempts = 0; attempts < RETRYMAX; attempts++)
+    {
+      sendbyte (smallhdr ? SOH : stx_soh);
+      sendbyte (sectnum);
+      sendbyte (-sectnum - 1);
+      for (j = 0; j < (smallhdr ? 128 : ksecsize); j++)
+	sendbyte (wcbuf[j]);
+
+      /* 
+         Wait for output to drain before sending checksum/crc.
+         Clear input buffer of any noise in anticipation of an
+         acknowledgement after we send the checksum/crc. 
+       */
+
+      xc_drain (DRAIN_FLAG);
+
+      if (crcheck)
+	{
+	  sendbyte ((int) (oldcrc >> 8));
+	  sendbyte ((int) oldcrc);
+	}
+      else
+	sendbyte (checksum);
+
+      /* check for response */
+      if (firstsec && y_flag)
+	{
+	  if (g_flag)
+	    {			/* Ymodem-g */
+	      int ack_ct = 12;	/* arbitrary max ACKS before error */
+	      int timeout_ct = 4;	/* arbitrary max number of timeouts */
+	      if (y_done)
+		{
+		  purge ();	/* delete any extraneous trailing ACKs */
+		  return OK;
+		}
+	      do
+		{
+		  c = readbyte (10);
+		  if (c == WANTCRG || c == CAN)
+		    break;
+		  else if (c == TIMEOUT)
+		    timeout_ct--;
+		  else
+		    c = TIMEOUT;
+		}
+	      while (--ack_ct && timeout_ct);
+	      /* according to Forsberg any response other than the following
+	         is undefined at this sequence point in the ymodem-g protocol:  
+	       */
+	      assert (c == TIMEOUT || c == WANTCRG || c == CAN);
 	    }
 	  else
-	      sendbyte (checksum);
-	
-	/* check for response */
-	if( firstsec && y_flag )
+	    {			/* normal non-streaming ymodem */
+	      c = readbyte (10);
+	      if (c == ACK)
+		{
+		  firstsec = smallhdr = FALSE;
+		  if (y_done)
+		    return OK;
+		  c = readbyte (20);
+		}
+	    }
+	  if (crcheck && c == crcheck)
+	    {
+	      assert (c == WANTCRC || c == WANTCRG);
+	      firstsec = smallhdr = FALSE;
+	      return OK;
+	    }
+	}
+      else
 	{
-		c = readbyte(10);
-		/*if( c == (g_flag ? WANTCRG : ACK ))*/
-		if( c == ACK ){
-		   firstsec = smallhdr = FALSE ;
-		   if( y_done ) return OK ;
-		   c = readbyte(20);
-		   if ( crcheck && c == crcheck )
-		   	return OK ;
-		} 
-		
-	} else {
-	  c = readbyte( g_flag ? -1 : 10);
+	  c = readbyte (g_flag ? RB_POLL : 10);
 	}
 
-	 /* detect false CAN, require 2 in a row */
-	  if ( c == CAN )
-	  	c = readbyte( g_flag ? 1 : 10);
-	  if ( g_flag && c != CAN && ( !firstsec || !y_flag )) 
-	  	return OK ;
+      /* detect false CAN, require 2 in a row */
+      if (c == CAN)
+	c = readbyte (g_flag ? 1 : 10);
+      if (g_flag && c != CAN && (!firstsec || !y_flag))
+	return OK;
 
-	  switch ( c )
+      switch (c)
+	{
+	case CAN:
+	  show (2, "Receiver CANcelled");
+	  return (ERROR);
+	case NAK:
+	  show (2, "Got a NAK on sector acknowledge ");
+	  break;
+	case TIMEOUT:
+	  show (2, "Timeout on sector acknowledge   ");
+	  break;
+	case ACK:
+	  assert (g_flag == FALSE);
+	  firstsec = FALSE;
+	  return (OK);
+	  break;
+	default:
+	  sprintf (Msg, "Got %#x for sector acknowledge ", c);
+	  show (2, Msg);
+
+	  if (c == crcheck && crcheck)
+	    break;		/* imitate NAK */
+	  do
 	    {
-	    case CAN:
-		show(2,"Receiver CANcelled");
-		return (ERROR);
-	    case ACK:
-		firstsec = FALSE;
-		return (OK);
-	    case NAK:
-		show(2,"Got a NAK on sector acknowledge ");
-		break;
-	    case TIMEOUT:
-		show(2,"Timeout on sector acknowledge   ");
-		break;
-	    default:
-		sprintf(Msg,"Got %#x for sector acknowledge ", c);
-		show(2,Msg);
-		if( c == crcheck && crcheck ) break ; /* imitate NAK */
-		do
-		  {
-		      if ((c = readbyte (3)) == CAN)
-			{
-			    show(2,"Receiver CANcelled");
-			    return (ERROR);
-			}
-		  }
-		while (c != TIMEOUT);
-		break;
+	      if ((c = readbyte (3)) == CAN)
+		{
+		  show (2, "Receiver CANcelled");
+		  return (ERROR);
+		}
 	    }
-	  errct++;
-	  if ( !y_flag && (errct == 3) && (badline == FALSE || stx_soh == STX))
-	    {
-		show (2, "ATTEMPTING RECOVERY ");
-		return (FALLBACK);
-	    }
-	  
-      }
-    show(2,"Retry count exceeded");
-    return (ERROR);
+	  while (c != TIMEOUT);
+	}
+      errct++;
+      if (!y_flag && (errct == 3) && (badline == FALSE || stx_soh == STX))
+	{
+	  show (2, "ATTEMPTING RECOVERY ");
+	  return (FALLBACK);
+	}
+
+    }
+  show (2, "Retry count exceeded");
+  return (ERROR);
 }
 
 /* Transmit a file using XMODEM protocol */
 static int
-wctx ()
+wctx P_((void))
 {
     register c ;
     int sectnum, eoflg, attempts;
@@ -660,33 +689,14 @@ setmode (c)
     return SUCCESS;
 }
 
-/*      Put received WC sector into a UNIX file
-        using text translations if neccesary.
-*/
-
-void
-xreceive (c)
-     int c;
-{
-    oldvec = signal (SIGINT, xmsigint);
-
-    if (setmode (c))
-      {
-	  if (!setjmp (our_env))
-	    {
-		if (wcrx () == ERROR)
-		    canit ();
-		/* return; why isn't signal restored first? *??*/
-	    }
-      }
-
-    signal (SIGINT, oldvec);
-}
+/* save old signal handler */
+static RETSIGTYPE (*oldvec) P_((int));
 
 void
 xsend (c)
      int c;
 {
+    badline = FALSE ;
     oldvec = signal (SIGINT, xmsigint);
 
     if (setmode (c))
@@ -710,60 +720,16 @@ xsend (c)
 }
 
 
-/* automatically negotiate crc or checksum reception */
-
-char
-syncem (void)
-{
-    int ct = 2;
-    char c;
-
-    purge ();
-
-    while (ct)
-      {	  /* try to negotiate a CRC transfer */
-	  sendbyte ( g_flag ? WANTCRG : WANTCRC );
-	  c = readbyte (10);
-	  if (c != CAN && c != SOH && c != STX)
-	      ct--;
-	  else
-	    {
-		sendchar = ACK;
-		crcheck = g_flag ? WANTCRG : WANTCRC ;
-		return (c);
-	    }
-	    if ( g_flag ) { ct = 2, g_flag = FALSE ;} /* try again without "G" */
-      }
-    ct = 2;
-    purge ();
-    while (ct)
-      {				/* maybe we can initiate a checksum transfer */
-	  sendbyte (NAK);
-	  c = readbyte (10);
-	  if (c != CAN && c != SOH && c != STX)
-	      ct--;
-	  else
-	    {
-		sendchar = ACK;
-		crcheck = FALSE;
-		return (c);
-	    }
-      }
-    purge ();
-    return (TIMEOUT);
-
-}
-
-/******* Here is where most of the YMODEM code starts ******/
-
-/* normalize file name, assume *filn is at least 128 chars */
-char *
-getfname( char *filn, int strpflg )
+/* ymodem -- normalize file name, assume *filn is at least 128 chars */
+static char *
+getfname( filn, strpflg )
+  char *filn; 
+  int strpflg ;
 {
 	char *wtp, *utp ;
 
 	wtp = filn ;
-	utp = wtp + 128 ; /*!! debug: fix the 128 thingy */
+	utp = wtp + 128 ; /* length is 128 */
 
 	while ( *wtp == 0x20 ) { 	/* strip leading spaces */
 		wtp++ ;
@@ -790,104 +756,10 @@ getfname( char *filn, int strpflg )
 }
 
 
-int
-build_hdr()
-{
-	int retcode = 0, i = 0;
-	struct stat sbuf ;
-	char *fname, *tpr = wcbuf ;
-
-        while ( i < SECSIZ ) tpr[i] = 0, i++ ;  /* clear buffer */
-	getword();
-	fname = getfname( word, TRUE );
-
-	if ( fname )
-	{
-		if ( stat( word, &sbuf ))
-		{
-			perror( fname );
-			fprintf( stderr, "\rCan't stat()  %s\n\r", fname );
-			retcode = -1 ;
-		}
-		else
-		{
-			smallhdr = TRUE ;
-			strcpy( wcbuf, fname );
-			while( *tpr ) tpr++ ; tpr++ ; 
-			sprintf(tpr,"%ld %lo %o", sbuf.st_size, sbuf.st_mtime, sbuf.st_mode);
-			fprintf(stderr,"%s %ld %-25o\n\r", fname, sbuf.st_size, sbuf.st_mode);
-			while( *tpr ) tpr++ ;
-			if ( 128 < (tpr - wcbuf) ) smallhdr = FALSE ;
-		}	
-	}
-	else 
-	{
-		y_done = smallhdr = TRUE ;
-	}
-	return retcode ;
-}
-
-
-
-void
-ysend( int e )
-{
-    y_done = FALSE ;
-    oldvec = signal (SIGINT, xmsigint);
-
-    if (setmode ('b'))
-      {
-	
-	  g_flag = FALSE ; /* only receiver can set g_flag */
-	  sprintf (Msg, "Sending %s", ksecsize == 128 ? "128 byte small packet YMODEM"
-	          : "1K byte packets YMODEM  ") ; show(1,Msg) ;
-	  if (!setjmp (our_env))
-	    {
-		while( !y_done )
-		{
-			if( !build_hdr() ) 
-			{
-				if (wctx () == ERROR)
-		  		{
-		      		sprintf (Msg, "Error transmitting file %s", word);
-		      		show(1,Msg) ;
-		      		purge(); 
-				y_done = TRUE ;
-		      		/* return; why isn't signal restored? *??*/
-		  		}
-			}
-		}
-	    }
-      }
-    signal (SIGINT, oldvec);
-    g_flag = smallhdr = FALSE ;
-}
-
-/* re-set the date and perms of a file received via ymodem */
-void
-set_dp( const char *name_fl, const time_t time_fl, const unsigned long perm_fl )
-{
-	struct utimbuf utb ;
-
-	if ( name_fl != NULL ) {
-		if ( time_fl != 0 ) {
-			utb.actime = utb.modtime = time_fl ;
-			if ( utime( name_fl, &utb ))
-				perror(name_fl);
-		}
-		if ( perm_fl != 0 ) {
-			if ( chmod( name_fl, (mode_t) perm_fl ))
-				perror(name_fl);
-		}
-	}
-
-}
-
-
 /* parse ymodem header for file name and attributes */
 
 static int
-yparse()
+yparse P_((void))
 {
 	int i = 0 ;
 	char *wtp, *utp ;
@@ -926,18 +798,213 @@ yparse()
 }
 
 
+/* Receive a file using XMODEM protocol */
+static int
+wcrx P_((void))
+{
+    register sectnum, sectcurr;
+
+    if ( !y_flag ){
+    	/* open file for xmodem reception */
+    	strcpy (Name, word);
+    	xfp = QueryCreate (Resume_Not_Allowed) ;
+    	if ( xfp == NULL)
+		return (ERROR); 
+    }
+
+    firstsec = TRUE;
+    sectnum = y_flag ? -1 : 0 ;
+    sendchar = g_flag? WANTCRG : WANTCRC ;
+    sync = FALSE;
+    y_done = FALSE;
+    crcheck = TRUE;
+    show(2,"Sync...");
+
+    while (TRUE)
+      {
+	  if (badline)
+	      purge ();
+	  if ( g_flag == FALSE && sync ) /* g-mode doesn't ACK */
+	      sendbyte (sendchar);
+	  if ( y_done )
+	      { cl_line(); return OK; }
+	  /* 
+	     Here we are at this if() on the second pass through the loop.
+	     Here we start the ymodem transmission body after we have grabbed
+	     the intial header with file name. Presumably crcheck
+	     equals 'G' or 'C' or 0 at this juncture in time . . . 
+	  */
+	  if ( sectnum == 0 && y_flag ) {
+		assert(!crcheck || crcheck == WANTCRG || crcheck == WANTCRC);
+	        sendbyte((crcheck ? crcheck : NAK));
+	  }
+
+	  sectcurr = wcgetsec (Maxtime);
+
+	  if (sectcurr == ((sectnum + 1) & 0xff))
+	    {
+		if ( firstsec ){
+			if ( y_flag ){	/* open file for rx */
+				if (yparse() == ERROR ) return ERROR;
+			}
+			else{ 
+				fprintf(stderr,"\r%s\n\r", Name );
+			}
+			firstsec = FALSE ;
+		}
+		sectnum++;
+	        if (sectnum == 1)
+	            show (1,(ksecsize == 128 ? "Sender using 128 byte packets   ":"Sender using 1K byte packets    "));
+		if ( sectnum > 0 )
+		    putsec ();
+		fprintf (tfp, "Received sector #%d\r", sectnum);
+		sendchar = ACK;
+		continue;
+	    }
+
+	  if (sectcurr == (sectnum & 0xff))
+	    {
+		sprintf (Msg, "Received duplicate sector #%d", sectnum);
+		show(2,Msg);
+		sendchar = ACK;
+		continue;
+	    }
+
+	  fclose(xfp);
+	  if( y_flag ) set_dp( phdr.fnam, phdr.fdat, phdr.fprm );
+
+	  if (sectcurr == WCEOT)
+	    {
+		show( 1,"File received OK                ");
+		sendbyte (ACK);
+		return (OK);
+	    }
+
+	  if (sectcurr == ERROR)
+	      return (ERROR);
+
+	  sprintf (Msg, "Sync error ... expected %d(%d), got %d",
+		   (sectnum + 1) & 0xff, sectnum, sectcurr);
+	  show(1,Msg);
+	  return (ERROR);
+      }
+}
+
+
+/*      Put received WC sector into a UNIX file
+        using text translations if neccesary.
+*/
+
 void
-yreceive( int e )
+xreceive (c)
+     int c;
+{
+    badline = FALSE ;
+    oldvec = signal (SIGINT, xmsigint);
+
+    if (setmode (c))
+      {
+	  if (!setjmp (our_env))
+	    {
+		if (wcrx () == ERROR)
+		    canit ();
+		/* return; why isn't signal restored first? *??*/
+	    }
+      }
+
+    signal (SIGINT, oldvec);
+}
+
+static int
+build_hdr P_((void))
+{
+	int retcode = 0, i = 0;
+	struct stat sbuf ;
+	char *fname, *tpr = wcbuf ;
+
+        while ( i < SECSIZ ) tpr[i] = 0, i++ ;  /* clear buffer */
+	getword();
+	fname = getfname( word, TRUE );
+
+	if ( fname )
+	{
+		if ( stat( word, &sbuf ))
+		{
+			perror( fname );
+			fprintf(stderr, "\rCan't stat()  %s\n\r", fname );
+			retcode = -1 ;
+		}
+		else
+		{
+			smallhdr = TRUE ;
+			strcpy( wcbuf, fname );
+			while( *tpr ) tpr++ ; tpr++ ; 
+			sprintf(tpr,"%ld %lo %o", sbuf.st_size, sbuf.st_mtime, sbuf.st_mode);
+			fprintf(stderr,"%s %ld %-25o\n\r", fname, sbuf.st_size, sbuf.st_mode);
+			while( *tpr ) tpr++ ;
+			if ( 128 < (tpr - wcbuf) ) smallhdr = FALSE ;
+		}	
+	}
+	else 
+	{
+		y_done = smallhdr = TRUE ;
+	}
+	return retcode ;
+}
+
+
+
+void
+ysend P_((void))
 {
     y_done = FALSE ;
+    badline = FALSE ;
+    oldvec = signal (SIGINT, xmsigint);
+
+    if (setmode ('b'))
+      {
+	
+	  g_flag = FALSE ; /* only receiver can set g_flag */
+	  sprintf (Msg, "Sending %s", ksecsize == 128 ? "128 byte small packet YMODEM"
+	          : "1K byte packets YMODEM  ") ; show(1,Msg) ;
+	  if (!setjmp (our_env))
+	    {
+		while( !y_done )
+		{
+			if( !build_hdr() ) 
+			{
+				if (wctx () == ERROR)
+		  		{
+		      		sprintf (Msg, "Error transmitting file %s", word);
+		      		show(1,Msg) ;
+		      		purge(); 
+				y_done = TRUE ;
+		      		/* return; why isn't signal restored? *??*/
+		  		}
+			}
+		}
+	    }
+	    purge(); 
+      }
+    signal (SIGINT, oldvec);
+    g_flag = smallhdr = FALSE ;
+}
+
+
+void
+yreceive( e )
+ int e ;
+{
+    y_done = FALSE ;
+    badline = FALSE ;
     oldvec = signal (SIGINT, xmsigint);
     
-    purge(); /*!!*/
+    purge();
     if (setmode ('b'))
     {
 	  if (!setjmp (our_env))
 	    {
-    		g_flag = e == 'g' ? TRUE : FALSE ; 
+    		g_flag = (e == 'g' ? TRUE : FALSE ); 
 		while( y_done == FALSE )
 		{
 			if (wcrx () == ERROR)
